﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;			//SerialPort
using System.Globalization;		//NumberStyles

class	CdcIO	:	SerialPort
{
	public bool cmdGetError = false;
	
	//コンストラクタ
	//デフォルト設定：COM1, 9600bps, Parity.NONE, Data=8bit, StopBits.One
	public CdcIO() : base()
	{ }
	
	//ポートをオープンする
	public bool Open(string portName)
	{
		PortName = portName;
		try
		{
			base.Open();
		}
		catch { return false; }
		return IsOpen;
	}

	public int Get(SFR addr)
	{
		int n = 0;
		cmdGetError = false;
		try
		{
			WriteLine(String.Format("{0:x2} ?", (int)addr));
			n = Int32.Parse(ReadLine(), NumberStyles.HexNumber);
		}
		catch { cmdGetError = true; }
		return (n & 0xFF);
	}
	
	public bool Set(SFR addr, int data)
	{
		try
		{
			WriteLine(String.Format("{0:x2} {1:x2} =", data & 0xFF, (int)addr));
			ReadLine();
		}
		catch { return false; }
		return true;
	}
	
	public bool SetXor(SFR addr, int data)
	{
		try
		{
			WriteLine(String.Format("{0:x2} {1:x2} ^", data & 0xFF, (int)addr));
			ReadLine();
		}
		catch { return false; }
		return true;
	}

	/*	ATtiny2313では非対応
	//	Obtain device name
	public string Who()
	{
		string s = null;
		try
		{
			WriteLine("@");
			s = ReadLine();
		}
		catch { return null; }
		return s;
	}
	
	public bool SetAnd(SFR addr, int data)
	{
		try
		{
			WriteLine(String.Format("{0:x2} {1:x2} &", data & 0xFF, (int)addr));
			ReadLine();
		}
		catch { return false; }
		return true;
	}
	
	public bool SetOr(SFR addr, int data)
	{
		try
		{
			WriteLine(String.Format("{0:x2} {1:x2} |", data & 0xFF, (int)addr));
			ReadLine();
		}
		catch { return false; }
		return true;
	}
	*/
	
	//シリアル通信初期化（AVR-CDCのRXD/TXDを使う準備）
	public bool initSerial()
	{
		//12MHz/4800bps -> N=155(.25) -> 4807.7 (0.16%)
		//UMSEL=0:非同期, UPM1/0=00:パリティなし, USBS=0:ストップビット1, UCSRB:UCSZ2-UCSRC:UCSZ1/0=011:データ8ビット
		
		bool ret = true;
		ret &= Set(SFR.UBRRH, 0);
		ret &= Set(SFR.UBRRL, 155);
		ret &= Set(SFR.UCSRB, 1 << RB.RXEN | 1 << RB.TXEN | 0 << RB.UCSZ2);		//RXD(PD0),TXD(PD1)を有効にする
		ret &= Set(SFR.UCSRC, 0 << RB.UMSEL | 0 << RB.UPM1 | 0 << RB.UPM0 | 0 << RB.USBS | 1 << RB.UCSZ1 | 1 << RB.UCSZ0);
		return ret;
	}
	
	//データを1byte送信する
	public bool sendByte(int data)
	{
		while ((Get(SFR.UCSRA) & 1 << RB.UDRE) == 0) ;	//USART送信データレジスタ空きフラグ
		return Set(SFR.UDR, data);
	}

	//データを1byte受信する
	//接続先デバイスにトラブルが発生して通信不能になった場合の対策をしておく
	//（USART受信完了フラグが立たないときの永久ループ回避策を入れる）
	public int receiveByte()
	{
		cmdGetError = false;
		int	count = 0;
		while (	++count < 1000	&&	(Get(SFR.UCSRA) & 1 << RB.RXC) == 0	) ;	//USART受信完了フラグ
		if (count == 1000)
		{
			cmdGetError = true;
			return 0;
		}
		return Get(SFR.UDR);
	}
	
}
