﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
//
using System.IO.Ports;	//SerialPort
using System.Threading;	//Sleep()

namespace LCDCtrlTest
{
	public partial class FormMain : Form
	{
		private	CdcIO	cdcio;
		private LcdLib	lcd;
		private	bool	isConnect;
		
		//コンストラクタ
		public FormMain()
		{
			InitializeComponent();
			this.Text = "LCD Control Test v1.00";
			lblAuthor.Text = "『昼夜逆転』工作室" + Environment.NewLine + "J.Sugita";
			
			cdcio = new CdcIO();
			lcd = new LcdLib();
			isConnect = false;
			
			//COMポートを探す
			if (comPortListup())
			{
				setControlEnabled();
				cbCOMPortList.Enabled = true;
				btnConnect.Enabled = true;
				info("COMポートを指定し, 接続してください.");
			}
			else
			{
				info("COMポートが見つかりません.");
			}
		}
		
		//現在の状態やエラーメッセージを表示する
		public void info(string mes)
		{
			lblInfo.Text = mes;
		}

		//COMポートを選択リストにセットする
		private bool comPortListup()
		{
			cbCOMPortList.Items.Clear();
			foreach (string name in SerialPort.GetPortNames())
				cbCOMPortList.Items.Add(name);

			if (cbCOMPortList.Items.Count == 0)
				return false;

			cbCOMPortList.SelectedIndex = cbCOMPortList.Items.Count - 1;
			return true;
		}

		private void btnConnect_Click(object sender, EventArgs e)
		{
			connectAction();
			if (isConnect) displayDemo();
		}
		
		private void connectAction()
		{
			if (isConnect)
			{
				if (!doDisconnect()) return;

				//コントロールの有効状態を設定する
				setControlEnabled();
			}
			else
			{
				if (!doConnect())
				{
					setControlEnabled();
					return;
				}

				//シリアル通信初期化（AVR-CDCのRXD/TXDを使う準備）
				if (!cdcio.initSerial())
				{
					//AVR-CDCではないCOMポートへ接続している可能性が高い
					doDisconnect();
					info("ポートが違うか, AVR-CDCが動作していません.");
					return;
				}
				
				//LcdLibにcdcioをセットする
				//これ以降、LCDを操作してよい
				lcd.setCdcIO(cdcio);
				
				//コントロールの有効状態を設定する
				setControlEnabled();
			}
		}

		//コントロールのenabled/disabledを設定する
		private void setControlEnabled()
		{
			cbCOMPortList.Enabled = !isConnect;
			btnConnect.Text = isConnect ? "解除" : "接続";
		}

		//接続開始する
		private bool doConnect()
		{
			string portName = (string)cbCOMPortList.SelectedItem;
			if (!cdcio.Open(portName))
			{
				info(portName + "のオープン失敗.");
				return false;
			}
			
			isConnect = true;
			info(portName + "に接続しました.");
			return true;
		}

		//接続解除する
		private bool doDisconnect()
		{
			bool error = false;
			string portName = (string)cbCOMPortList.SelectedItem;
			
			try
			{
				cdcio.Close();
			}
			catch
			{
				error = true;
				info(portName + "のクローズ失敗.");
			}
			finally
			{
				try
				{
					cdcio.Dispose();
				}
				catch
				{
					if (!error)
					{
						error = true;
						info(portName + "の破棄失敗.");
					}
				}
			}

			isConnect = cdcio.IsOpen;
			if (isConnect)
				error = true;
			else
				info(portName + "を接続解除しました.");

			return !error;
		}
		
		//表示デモ
		private	void	displayDemo()
		{
			//７６５４３２１０
			//＿＿＿■■■□□
			//＿＿＿■□□■□
			//＿＿＿■■■□□
			//＿＿＿■□□□□
			//＿＿＿□□■■■
			//＿＿＿□■□□□
			//＿＿＿□■□□□
			//＿＿＿□□■■■
			byte[] dots = { 0x1C, 0x12, 0x1C, 0x10, 0x07, 0x08, 0x08, 0x07 };
			
			lcd.init();
			lcd.defChar(0, dots);	//文字コード0番に定義

			lcd.setPos(0, 0);
			lcd.putStr(@"12345678901234567890");
			lcd.setPos(0, 1);
			lcd.putStr(@"ABCDEｱｲｳｴｵabcdeｶｷｸｹｺ");
			Thread.Sleep(2000);
			lcd.cls();
			lcd.setPos(0, 0);
			lcd.putStr(@"!""#$%&'(=^|@{[;*:<?/");
			lcd.setPos(0, 1);
			lcd.putChar((char)0);	//dots[]で定義した文字
			lcd.setPos(2, 1);
			lcd.putInt(Int32.MinValue);
		}

	}
}
